#!/bin/bash
#
#  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
#
#  This file is part of the TerraLib - a Framework for building GIS enabled applications.
#
#  TerraLib is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TerraLib is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TerraLib. See COPYING. If not, write to
#  TerraLib Team at <terralib-team@terralib.org>.
#
#  Description: TerraLib Layout Package for Linux Ubuntu environment.
#
#  Before start, please read the "PACKAGE-INSTRUCTIONS".
#

#
# Set package info:
#
DEBNAME=terralib-layout
DEBVERSION=1.2.0
DEBARC=amd64
LIBNAME=Layout
FOLDERNAME=terraprint
FILENAME=${FOLDERNAME}-source.tar.gz
GIT_URL=https://gitlab.dpi.inpe.br/terralib/terraprint

if [ -z "$INSTALL_DIR" ]; then
  INSTALL_DIR=/opt/terraprint/1.1.0
fi

if [ -z "$OUTPUT_PATH" ]; then
  OUTPUT_PATH=`pwd`/../../../3rdparty
fi

#
# Check if the package already exist in the "OUTPUT_PATH":
#
if [ ! -f "${OUTPUT_PATH}/packages/${DEBNAME}_${DEBVERSION}_${DEBARC}.deb" ]; then
#
# Unpack source:
#
if [ ! -d "${FOLDERNAME}" ]; then
  if [ ! -f "${FILENAME}" ]; then
    git clone -b develop ${GIT_URL}.git
  else
    tar xzvf ${FILENAME}
  fi
fi
#
# Create debian folder:
#
cd ${FOLDERNAME}
rm -rf debian
mkdir -p debian
#
# Create the copyright file:
#
cat > debian/copyright <<EOF
Format: http://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
Upstream-Name: TerraLib ${LIBNAME} Library
Upstream-Contact: TerraLib Team <terralib-team@terralib.org>
Source: https://gitlab.dpi.inpe.br/terralib/terralib

Files: *
Copyright: Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil
License: LGPL-3.0
  TerraLib is free software, you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or 
  (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU General Public License  
  along with this program. If not, write to TerraLib Team at <terralib-team@terralib.org>.
EOF
#
# Create the changelog (no messages needed):
#
dch --create -v ${DEBVERSION} --distribution unstable --package ${DEBNAME} ""
#
# Create control file:
#
cat > debian/control <<EOF
Source: ${DEBNAME}
Maintainer: TerraLib Team <terralib-team@terralib.org>
Section: misc
Priority: optional
Standards-Version: 3.9.2
Build-Depends: debhelper (>= 7)

Package: ${DEBNAME}
Architecture: ${DEBARC}
Depends: \${shlibs:Depends}, \${misc:Depends}, ${DEBNAME} (= ${DEBVERSION})
Description: TerraLib ${LIBNAME} Library, version ${DEBVERSION}
Replaces:${DEBNAME} (>= 1.2)
EOF
#
# Create rules file:
#
cat > debian/rules <<EOF 
#!/usr/bin/make -f
%:
	dh \$@
override_dh_shlibdeps:
	dh_shlibdeps --dpkg-shlibdeps-params=--ignore-missing-info
override_dh_auto_configure:
	mkdir -p debian/${DEBNAME}${INSTALL_DIR}
	cmake -G "Unix Makefiles" -DCMAKE_BUILD_TYPE:STRING="Release" -DTERRALIB_TRACK_3RDPARTY_DEPENDENCIES:BOOL=ON -DCMAKE_PREFIX_PATH:PATH="/opt/terralib/$TE_VERSION/3rdparty;/opt/terralib/$TE_VERSION/3rdparty/gdal2" -Dterralib_DIR:PATH="/opt/terralib/$TE_VERSION/lib/cmake/terralib-$TE_VERSION" -DTERRALIB_BUILD_EXAMPLES_ENABLED:BOOL=OFF -DTERRALIB_BUILD_LAYOUT_TEST_ENABLED:BOOL=OFF -DCMAKE_INSTALL_PREFIX:PATH="`pwd`/debian/${DEBNAME}${INSTALL_DIR}" -DCMAKE_INSTALL_RPATH:PATH="`pwd`/debian/${DEBNAME}${INSTALL_DIR}/lib" -DCMAKE_SKIP_BUILD_RPATH:BOOL="OFF" -DCMAKE_BUILD_WITH_INSTALL_RPATH:BOOL="OFF" -DCMAKE_INSTALL_RPATH_USE_LINK_PATH:BOOL="ON" build/cmake
override_dh_auto_build:
	PREFIX=`pwd`/debian/${DEBNAME}${INSTALL_DIR} make -j 4
override_dh_auto_test:
override_dh_auto_install:
	PREFIX=`pwd`/debian/${DEBNAME}${INSTALL_DIR} make install -j 4
EOF
#
# Create postinst and postrm file:
#
cat > debian/postinst <<EOF 
#!/bin/sh
sudo cp ${INSTALL_DIR}/lib/libterralib_mod_layout.so ${INSTALL_DIR}/lib/libterralib_mod_qt_plugins_layout.so /opt/terralib/${TE_VERSION}/lib
sudo cp ${INSTALL_DIR}/lib/libterralib_mod_layout.so.1.1 ${INSTALL_DIR}/lib/libterralib_mod_qt_plugins_layout.so.1.1 /opt/terralib/${TE_VERSION}/lib
sudo ln -sf ${INSTALL_DIR}/share/terralib/plugins/te.qt.layout.teplg.json /opt/terralib/${TE_VERSION}/share/terralib/plugins/te.qt.layout.teplg.json
EOF

cat > debian/postrm <<EOF 
#!/bin/sh
sudo rm /opt/terralib/${TE_VERSION}/lib/libterralib_mod_layout.so /opt/terralib/${TE_VERSION}/lib/libterralib_mod_qt_plugins_layout.so /opt/terralib/${TE_VERSION}/share/terralib/plugins/te.qt.layout.teplg.json
EOF

chmod 0755 debian/postinst debian/postrm
#
# Create some misc files:
#
echo "8" > debian/compat
mkdir -p debian/source
echo "3.0 (quilt)" > debian/source/format
#
# Build the package:
#
# To add a GnuPG key just uncomment the "-k" and change the following key.
nice -n19 ionice -c3 debuild -us -uc -b #-kFBC36213
#
# Clean debian-package directory?
#
cd ..
if [ -f "${DEBNAME}_${DEBVERSION}_${DEBARC}.deb" ]; then
  rm -rf ${FOLDERNAME}
  mkdir -p ${OUTPUT_PATH}/packages ${OUTPUT_PATH}/build-changes
  mv ${DEBNAME}_${DEBVERSION}_${DEBARC}.deb ${OUTPUT_PATH}/packages
  mv ${DEBNAME}_${DEBVERSION}_${DEBARC}.changes ${OUTPUT_PATH}/build-changes
  mv ${DEBNAME}_${DEBVERSION}_${DEBARC}.build ${OUTPUT_PATH}/build-changes
fi
fi