#!/bin/bash
#
#  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
#
#  This file is part of the TerraLib - a Framework for building GIS enabled applications.
#
#  TerraLib is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TerraLib is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TerraLib. See COPYING. If not, write to
#  TerraLib Team at <terralib-team@terralib.org>.
#
#  Description: TerraLib Package for Linux Ubuntu environment.
#
#  Before start, please read the "PACKAGE-INSTRUCTIONS".
#

# Valid parameter val or abort script
function valid_operation()
{
  if [ $1 -ne 0 ]; then
    echo $2
    echo ""
    exit $1
  else
    echo "done."
  fi
}

# Defining value to variables ROOT_DIR and CODEBASE
ROOT_DIR=`pwd`
CODEBASE=${ROOT_DIR}/../..


# Defining default value to OUTPUT_PATH if not set
if [ -z "$OUTPUT_PATH" ]; then
  OUTPUT_PATH=${CODEBASE}/../3rdparty
fi

# Defining install and build variables
export TE_INSTALL=/opt/terralib/${TE_VERSION}

if [ -z "$TE_BUILD_DIR" ]; then
  TE_BUILD_DIR=${CODEBASE}/../build-package
fi


mkdir -p ${TE_BUILD_DIR}
cp terralib.conf.cmake ${TE_BUILD_DIR}
cd ${TE_BUILD_DIR}

# Generating Makefiles
cmake -G "Unix Makefiles" -C ./terralib.conf.cmake -DCMAKE_BUILD_TYPE="Release" ${CODEBASE}/build/cmake
valid_operation $? "Error: could not generate TerraLib makefiles"

# Building TerraLib
LD_LIBRARY_PATH=${TE_BUILD_DIR}/bin make package -j 8
valid_operation $? "Error: could not make TerraLib"

# Saving files in output path
if [ -f "${TE_BUILD_DIR}/terralib-${TE_VERSION}.deb" ]; then
	cd ${ROOT_DIR}
	cp ${TE_BUILD_DIR}/terralib-${TE_VERSION}.deb  ${OUTPUT_PATH}/packages
fi