/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/unittest/rp/functions/TsRasterAttributes.cpp

  \brief A test suit for the Functions interface.
*/

// TerraLib
#include "../Config.h"
#include <terralib/rp.h>
#include <terralib/raster.h>
#include <terralib/dataaccess.h>

// STL
#include <memory>
#include <string>

// Boost
#define BOOST_TEST_NO_MAIN
#include <boost/test/unit_test.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/timer.hpp>

BOOST_AUTO_TEST_SUITE (raster_attributes_tests)

BOOST_AUTO_TEST_CASE( getGLCMMetrics_test )
{
  /* Openning input raster */

  std::map<std::string, std::string> auxRasterInfo;

  auxRasterInfo["URI"] = TERRALIB_DATA_DIR "/geotiff/cbers_rgb342_crop1.tif";
  boost::shared_ptr< te::rst::Raster > inputRasterPtrPointer ( te::rst::RasterFactory::open(
    auxRasterInfo ) );
  BOOST_CHECK( inputRasterPtrPointer.get() );

  // raster_attributes_tests

  te::rp::RasterAttributes attributesInstance( true );

  boost::numeric::ublas::matrix<double> glcm;
  glcm = attributesInstance.getGLCM( *inputRasterPtrPointer, 0, 1, 1, 0, 255, 256 );

  te::rp::Texture glcmMetrics = attributesInstance.getGLCMMetrics( glcm );

  BOOST_REQUIRE_CLOSE( glcmMetrics.m_contrast, 4.0108956721513795, 0.00001 );
  BOOST_REQUIRE_CLOSE( glcmMetrics.m_dissimilarity, 1.3006917202426418, 0.00001 );
  BOOST_REQUIRE_CLOSE( glcmMetrics.m_energy, 0.098389689308334138, 0.00001 );
  BOOST_REQUIRE_CLOSE( glcmMetrics.m_entropy, 5.1017890175144585, 0.00001 );
  BOOST_REQUIRE_CLOSE( glcmMetrics.m_homogeneity, 0.53576701787364245, 0.00001 );
}

BOOST_AUTO_TEST_SUITE_END()
